//---------------------------------------------------------------------------


#pragma hdrstop

#include "Settings.h"
#include <algorithm>
#include <fstream> 
#include <json/json.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)

Settings appSettings;

void Settings::SetDefault(void)
{
	frmMain.iWidth = 350;
	frmMain.iHeight = 300;
	frmMain.iPosX = 30;
	frmMain.iPosY = 30;
	frmMain.bWindowMaximized = false;
	frmMain.bAlwaysOnTop = false;

	Logging.bLogToFile = false;
	Logging.iMaxUiLogLines = 5000;

	uaConf.accounts.clear();

	struct UaConf::Account new_acc;
	uaConf.accounts.push_back(new_acc);
}

int Settings::Read(AnsiString asFileName)
{
	Json::Value root;   // will contains the root value after parsing.
	Json::Reader reader;

    SetDefault();

	try
	{
		std::ifstream ifs(asFileName.c_str());
		std::string strConfig((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
		ifs.close();
		bool parsingSuccessful = reader.parse( strConfig, root );
		if ( !parsingSuccessful )
		{
			return 2;
		}
	}
	catch(...)
	{
		return 1;
	}

	const Json::Value uaConfJson = root["uaConf"];
	uaConf.soundBackend = (enum UaConf::SoundBackend)uaConfJson.get("soundBackend", UaConf::PORTAUDIO_DSOUND).asInt();
	if (uaConf.soundBackend < 0 || uaConf.soundBackend >= UaConf::SOUND_BACKEND_LIMIT)
	{
		uaConf.soundBackend = UaConf::PORTAUDIO_DSOUND;
	}

	uaConf.logMessages = uaConfJson.get("logMessages", false).asBool();
	uaConf.local = uaConfJson.get("localAddress", "").asString();

	uaConf.codecs.g711 = uaConfJson["Codecs"].get("G.711", true).asBool();
	uaConf.codecs.g726_32 = uaConfJson["Codecs"].get("G.726-32", false).asBool();
	uaConf.codecs.gsm = uaConfJson["Codecs"].get("GSM", false).asBool();
	uaConf.codecs.speex = uaConfJson["Codecs"].get("Speex", false).asBool();

	const Json::Value accounts = root["Accounts"];
	//uaConf.accounts.clear();
	for (int i=0; i<std::min(1u, accounts.size()); i++)
	{
		const Json::Value acc = accounts[i];
		struct UaConf::Account new_acc;

		new_acc.reg_server = acc.get("reg_server", "").asString();
		new_acc.user = acc.get("user", "").asString();
		new_acc.auth_user = acc.get("auth_user", "").asString();
		new_acc.pwd = acc.get("pwd", "").asString();
		new_acc.transport =
			(UaConf::Account::TRANSPORT_TYPE)acc.get("transport", UaConf::Account::TRANSPORT_UDP).asInt();
		if (new_acc.transport < 0 || new_acc.transport >= UaConf::Account::TRANSPORT_LIMITER)
		{
			new_acc.transport = UaConf::Account::TRANSPORT_UDP;
		}
		new_acc.reg_expires = acc.get("reg_expires", 60).asInt();
		if (new_acc.reg_expires < 0 || new_acc.reg_expires > 7200)
		{
			new_acc.reg_expires = 60;
		}

		new_acc.stun_server = acc.get("stun_server", "").asString();

		uaConf.accounts[0] = new_acc;
	}

	const Json::Value callHistory = root["CallHistory"];
	uaConf.callHistory.clear();
	for (int i=0; i<std::min((unsigned int)UaConf::CALL_HISTORY_LIMIT, callHistory.size()); i++)
	{
		const Json::Value call = callHistory[i];
		struct UaConf::HistoryEntry entry;

		entry.uri = call.get("uri", "").asString();
		uaConf.callHistory.push_back(entry);
	}

	int maxX = GetSystemMetrics(SM_CXSCREEN);
	int maxY = GetSystemMetrics(SM_CYSCREEN);

	const Json::Value frmMainJson = root["frmMain"];
	frmMain.iWidth = frmMainJson.get("AppWidth", 350).asInt();
	frmMain.iHeight = frmMainJson.get("AppHeight", 300).asInt();
	if (frmMain.iWidth < 250 || frmMain.iWidth > maxX + 20)
	{
		frmMain.iWidth = 250;
	}
	if (frmMain.iHeight < 200 || frmMain.iHeight > maxY + 20)
	{
		frmMain.iHeight = 200;
	}
	frmMain.iPosX = frmMainJson.get("AppPositionX", 30).asInt();
	frmMain.iPosY = frmMainJson.get("AppPositionY", 30).asInt();
	if (frmMain.iPosX < 0)
		frmMain.iPosX = 0;
	if (frmMain.iPosY < 0)
		frmMain.iPosY = 0;
	if (frmMain.iPosX + frmMain.iWidth > maxX)
		frmMain.iPosX = maxX - frmMain.iWidth;
	if (frmMain.iPosY + frmMain.iHeight > maxY)
		frmMain.iPosY = maxY - frmMain.iHeight;
	frmMain.bWindowMaximized = frmMainJson.get("Maximized", false).asBool();
	frmMain.bAlwaysOnTop = frmMainJson.get("AlwaysOnTop", false).asBool();

	const Json::Value LoggingJson = root["Logging"];
	Logging.bLogToFile = LoggingJson.get("LogToFile", false).asBool();
	Logging.iMaxUiLogLines = LoggingJson.get("MaxUiLogLines", 5000).asInt();

	return 0;
}

int Settings::Write(AnsiString asFileName)
{
    Json::Value root;
	Json::StyledWriter writer;

	root["frmMain"]["AppWidth"] = frmMain.iWidth;
	root["frmMain"]["AppHeight"] = frmMain.iHeight;
	root["frmMain"]["AppPositionX"] = frmMain.iPosX;
	root["frmMain"]["AppPositionY"] = frmMain.iPosY;
	root["frmMain"]["Maximized"] = frmMain.bWindowMaximized;
	root["frmMain"]["AlwaysOnTop"] = frmMain.bAlwaysOnTop;

	root["Logging"]["LogToFile"] = Logging.bLogToFile;
	root["Logging"]["MaxUiLogLines"] = Logging.iMaxUiLogLines;

	
	root["uaConf"]["soundBackend"] = uaConf.soundBackend;
	root["uaConf"]["logMessages"] = uaConf.logMessages;
	root["uaConf"]["localAddress"] = uaConf.local;

	root["uaConf"]["Codecs"]["G.711"] = uaConf.codecs.g711;
	root["uaConf"]["Codecs"]["G.726-32"] = uaConf.codecs.g726_32;
	root["uaConf"]["Codecs"]["GSM"] = uaConf.codecs.gsm;
	root["uaConf"]["Codecs"]["Speex"] = uaConf.codecs.speex;

	// write accounts
	for (unsigned int i=0; i<uaConf.accounts.size(); i++)
	{
		UaConf::Account &acc = uaConf.accounts[i];
		root["Accounts"][i]["reg_server"] = acc.reg_server;
		root["Accounts"][i]["user"] = acc.user;
		root["Accounts"][i]["auth_user"] = acc.auth_user;
		root["Accounts"][i]["pwd"] = acc.pwd;
		root["Accounts"][i]["transport"] = acc.transport;
		root["Accounts"][i]["reg_expires"] = acc.reg_expires;
		root["Accounts"][i]["stun_server"] = acc.stun_server;
	}

	for (unsigned int i=0; i<uaConf.callHistory.size(); i++)
	{
		UaConf::HistoryEntry &entry = uaConf.callHistory[i];
		root["CallHistory"][i]["uri"] = entry.uri;
	}

	std::string outputConfig = writer.write( root );

	try
	{
		std::ofstream ofs(asFileName.c_str());
		ofs << outputConfig;
		ofs.close();
	}
	catch(...)
	{
    	return 1;
	}

	return 0;
}
